@@ -24,9 +24,11 @@ import de.willuhn.datasource.GenericIterator;
 import de.willuhn.datasource.db.AbstractDBObjectNode;
 import de.willuhn.datasource.pseudo.PseudoIterator;
 import de.willuhn.datasource.rmi.DBIterator;
+import de.willuhn.datasource.rmi.DBObject;
 import de.willuhn.datasource.rmi.DBObjectNode;
 import de.willuhn.jameica.hbci.HBCI;
 import de.willuhn.jameica.hbci.rmi.Duplicatable;
+import de.willuhn.jameica.hbci.rmi.Konto;
 import de.willuhn.jameica.hbci.rmi.Umsatz;
 import de.willuhn.jameica.hbci.rmi.UmsatzTyp;
 import de.willuhn.jameica.messaging.StatusBarMessage;
@@ -150,10 +152,10 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
     DBIterator list = UmsatzUtil.getUmsaetze();
 
     if (von != null)
-      list.addFilter("datum >= ?", new Object[] {new java.sql.Date(von.getTime())});
-    
+      list.addFilter("datum >= ?", new java.sql.Date(von.getTime()));
+
     if (bis != null)
-      list.addFilter("datum <= ?", new Object[] {new java.sql.Date(bis.getTime())});
+      list.addFilter("datum <= ?", new java.sql.Date(bis.getTime()));
 
     if (this.isNewObject()) // Neuer Umsatztyp. Der hat noch keine ID
       list.addFilter("umsatztyp_id is null");
@@ -168,7 +170,7 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
       if (u.isAssigned() || matches(u)) // entweder fest zugeordnet oder passt via Suchfilter
         result.add(u);
     }
-    return PseudoIterator.fromArray((Umsatz[]) result.toArray(new Umsatz[result.size()]));
+    return PseudoIterator.fromArray((Umsatz[]) result.toArray(new Umsatz[0]));
   }
 
   /**
@@ -264,47 +266,96 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
         return false;
     }
 
+    ////////////////////////////////////////////////////////////////////////
+    // Zuordnung einer Kategorie zu einem Konto oder einer Kontogruppe
+    final String kat = this.getKontoKategorie();
+    final Konto k = this.getKonto();
+    if (k != null || (kat != null && kat.length() > 0))
+    {
+      final Konto test = umsatz.getKonto();
+      if (test != null) // sicher ist sicher
+      {
+        // Zuordnung zum Konto
+        if (k != null && !k.equals(test))
+          return false;
+        
+        // Zuordnung zur Gruppe
+        if (kat != null && !kat.equals(test.getKategorie()))
+          return false;
+      }
+    }
+    //
+    ////////////////////////////////////////////////////////////////////////
+
     String s = this.getPattern();
     if (s == null || s.trim().length() == 0)
       return false;
 
     String zweck = VerwendungszweckUtil.toString(umsatz,"");
     String name  = StringUtils.trimToEmpty(umsatz.getGegenkontoName());
+    String name2 = StringUtils.trimToEmpty(umsatz.getGegenkontoName2());
     String kto   = StringUtils.trimToEmpty(umsatz.getGegenkontoNummer());
     String kom   = StringUtils.trimToEmpty(umsatz.getKommentar());
     String art   = StringUtils.trimToEmpty(umsatz.getArt());
     String purp  = StringUtils.trimToEmpty(umsatz.getPurposeCode());
+    String ref   = StringUtils.trimToEmpty(umsatz.getCustomerRef());
     String e2eid = StringUtils.trimToEmpty(umsatz.getEndToEndId());
+    String mid   = StringUtils.trimToEmpty(umsatz.getMandateId());
+    String id    = StringUtils.trimToEmpty(umsatz.getID());
+    
+    // Im Suchbegriff knnen wir nicht nach Betrag suchen, da wir das Komma "," bereits als Trennzeichen
+    // fr mehrere Suchbegriffe verwenden. Es wre also nicht mglich, nach "50,00" zu suchen, weil dann
+    // auch alles gefunden werden wrde, wo "00" oder "50" drin steht.
     
     if (!isRegex())
     {
       zweck = zweck.toLowerCase();
       name  = name.toLowerCase();
+      name2 = name2.toLowerCase();
       kto   = kto.toLowerCase();
       kom   = kom.toLowerCase();
       art   = art.toLowerCase();
       purp  = purp.toLowerCase();
+      ref   = ref.toLowerCase();
       e2eid = e2eid.toLowerCase();
+      mid   = mid.toLowerCase();
 
       if (ignorewhitespace)
       {
         zweck = StringUtils.deleteWhitespace(zweck);
         name = StringUtils.deleteWhitespace(name); // BUGZILLA 1705 - auch im Namen koennen Leerzeichen sein
-        s = StringUtils.deleteWhitespace(s);
+        name2 = StringUtils.deleteWhitespace(name2);
       }
 
-      String[] list = s.toLowerCase().split(","); // Wir beachten Gross-Kleinschreibung grundsaetzlich nicht
+      String[] list = UmsatzTypUtil.splitQuery(s.toLowerCase(), ","); // Wir beachten Gross-Kleinschreibung grundsaetzlich nicht
 
       for (int i=0;i<list.length;++i)
       {
         String test = list[i].trim();
+        if (ignorewhitespace) {
+          test = StringUtils.deleteWhitespace(test);
+
+          /*
+           * While we already removed empty elements in split_escape(), this
+           * removal was only valid for actually empty elements.
+           * After whitespace removal, we might get additional empty elements
+           * (i.e., such that previously only contained whitespace).
+           */
+          if (test.isEmpty()) {
+              continue;
+          }
+        }
         if (zweck.indexOf(test) != -1 ||
             name.indexOf(test) != -1 ||
+            name2.indexOf(test) != -1 ||
             kto.indexOf(test)  != -1 ||
             kom.indexOf(test) != -1 ||
             art.indexOf(test) != -1 ||
             purp.indexOf(test) != -1 ||
-            e2eid.indexOf(test) != -1)
+            ref.indexOf(test) != -1 ||
+            e2eid.indexOf(test) != -1 ||
+            mid.indexOf(test) != -1 ||
+            id.equals(test))
         {
           return true;
         }
@@ -325,26 +376,32 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
       
       Matcher mZweck = pattern.matcher(zweck);
       Matcher mName = pattern.matcher(name);
+      Matcher mName2 = pattern.matcher(name2);
       Matcher mKto = pattern.matcher(kto);
       Matcher mKom = pattern.matcher(kom);
       Matcher mArt = pattern.matcher(art);
       Matcher mPurp = pattern.matcher(purp);
+      Matcher mRef = pattern.matcher(ref);
       Matcher mE2eid = pattern.matcher(e2eid);
-      Matcher mAll = pattern.matcher(name + " " + kto + " " + zweck + " " + kom + " " + art + " " + purp + " " + e2eid);
+      Matcher mMid = pattern.matcher(mid);
+      Matcher mAll = pattern.matcher(name + " " + name2 + " " + kto + " " + zweck + " " + kom + " " + art + " " + purp + " " + e2eid + " " + mid + " " + ref);
 
-      return (mZweck.matches() ||
-              mName.matches()  ||
-              mKto.matches()   ||
-              mKom.matches()   ||
-              mArt.matches()   ||
-              mPurp.matches()  ||
+      return (mAll.matches()    ||
+              mZweck.matches()  ||
+              mName.matches()   ||
+              mName2.matches()  ||
+              mKto.matches()    ||
+              mKom.matches()    ||
+              mArt.matches()    ||
+              mPurp.matches()   ||
+              mRef.matches()    ||
               mE2eid.matches()  ||
-              mAll.matches()
+              mMid.matches()
              );
     }
     catch (Exception e)
     {
-      Application.getMessagingFactory().sendMessage(new StatusBarMessage(i18n.tr("Ungltiger regulrer Ausdruck \"{0}\": {1}", new String[]{s,e.getMessage()}),StatusBarMessage.TYPE_ERROR));
+      Application.getMessagingFactory().sendMessage(new StatusBarMessage(i18n.tr("Ungltiger regulrer Ausdruck \"{0}\": {1}", s, e.getMessage()), StatusBarMessage.TYPE_ERROR));
       Logger.error("invalid regex pattern: " + e.getMessage(),e);
       return false;
     }
@@ -364,7 +421,7 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
    */
   public void setRegex(boolean regex) throws RemoteException
   {
-    setAttribute("isregex", new Integer(regex ? 1 : 0));
+    setAttribute("isregex", Integer.valueOf(regex ? 1 : 0));
   }
 
   @Override
@@ -445,6 +502,9 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
    */
   public Object getAttribute(String arg0) throws RemoteException
   {
+    if ("konto_id".equals(arg0))
+      return getKonto();
+
     // BUGZILLA 554
     if ("nummer-int".equals(arg0))
     {
@@ -453,7 +513,7 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
         return null;
       try
       {
-        Integer i = new Integer(n);
+        Integer i = Integer.valueOf(n);
         return i;
       }
       catch (Exception e)
@@ -464,10 +524,28 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
     }
     
     if ("umsatz".equals(arg0))
-      return new Double(getUmsatz());
+      return Double.valueOf(getUmsatz());
 
     return super.getAttribute(arg0);
   }
+  
+  /**
+   * @see de.willuhn.datasource.db.AbstractDBObject#overwrite(de.willuhn.datasource.rmi.DBObject)
+   */
+  public void overwrite(DBObject object) throws RemoteException
+  {
+    // Muessen wir ueberschreiben, weil wir fuer das Konto hier eine
+    // Sonderbehandlung machen. Wuerden wir das hier nicht machen,
+    // haetten wir nach dem Overwrite ploetzlich ein Konto-Objekt
+    // statt der Konto-ID in den Properties. Das liegt eigentlich
+    // nur daran, weil wir "konto_id" hier nicht als Foreign-Key
+    // (wegen dem Cache) deklariert haben - bei "getAttribute("konto_id")"
+    // aber trotzdem das Objekt (wegen KontoColumn) zurueckliefern.
+    super.overwrite(object);
+    
+    // Jetzt ersetzen wir wieder das Konto-Objekt gegen die ID
+    this.setKonto(((AbstractHibiscusTransferImpl)object).getKonto());
+  }
 
   /**
    * @see de.willuhn.datasource.db.AbstractDBObject#delete()
@@ -507,16 +585,11 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
       super.delete();
       transactionCommit();
     }
-    catch (RemoteException e)
+    catch (ApplicationException | RemoteException e)
     {
       this.transactionRollback();
       throw e;
     }
-    catch (ApplicationException e2)
-    {
-      this.transactionRollback();
-      throw e2;
-    }
   }
 
   /**
@@ -595,7 +668,7 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
    */
   public void setTyp(int typ) throws RemoteException
   {
-    setAttribute("umsatztyp",new Integer(typ));
+    setAttribute("umsatztyp", Integer.valueOf(typ));
   }
 
   /**
@@ -632,7 +705,7 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
    */
   public void setCustomColor(boolean b) throws RemoteException
   {
-    setAttribute("customcolor", new Integer(b ? 1 : 0));
+    setAttribute("customcolor", Integer.valueOf(b ? 1 : 0));
   }
 
   /**
@@ -665,60 +738,90 @@ public class UmsatzTypImpl extends AbstractDBObjectNode implements UmsatzTyp, Du
     t.setRegex(this.isRegex());
     t.setKommentar(this.getKommentar());
     t.setTyp(this.getTyp());
+    t.setKonto(this.getKonto());
+    t.setKontoKategorie(this.getKontoKategorie());
+    t.setFlags(this.getFlags());
     return t;
   }
+  
+  /**
+   * @see de.willuhn.jameica.hbci.rmi.UmsatzTyp#getKonto()
+   */
+  @Override
+  public Konto getKonto() throws RemoteException
+  {
+    Integer i = (Integer) super.getAttribute("konto_id");
+    if (i == null)
+      return null; // Kein Konto zugeordnet
+   
+    Cache cache = Cache.get(Konto.class,true);
+    return (Konto) cache.get(i);
+  }
+  
+  /**
+   * @see de.willuhn.jameica.hbci.rmi.UmsatzTyp#setKonto(de.willuhn.jameica.hbci.rmi.Konto)
+   */
+  @Override
+  public void setKonto(Konto konto) throws RemoteException
+  {
+    final Integer id = (konto == null || konto.getID() == null) ? null : Integer.valueOf(konto.getID());
+    setAttribute("konto_id",id);
+    
+    // Eine Zuordnung kann nur zu Konto ODER Kategorie moeglich sein - nicht beides gleichzeitig
+    if (id != null)
+      this.setKontoKategorie(null);
+  }
+  
+  /**
+   * @see de.willuhn.jameica.hbci.rmi.UmsatzTyp#getKontoKategorie()
+   */
+  @Override
+  public String getKontoKategorie() throws RemoteException
+  {
+    return (String) this.getAttribute("konto_kategorie");
+  }
+  
+  /**
+   * @see de.willuhn.jameica.hbci.rmi.UmsatzTyp#setKontoKategorie(java.lang.String)
+   */
+  @Override
+  public void setKontoKategorie(String kategorie) throws RemoteException
+  {
+    this.setAttribute("konto_kategorie",kategorie);
+    
+    // Eine Zuordnung kann nur zu Konto ODER Kategorie moeglich sein - nicht beides gleichzeitig
+    if (kategorie != null && kategorie.length() > 0)
+      this.setAttribute("konto_id",null);
+  }
+  
+  /**
+   * @see de.willuhn.jameica.hbci.rmi.Flaggable#hasFlag(int)
+   */
+  @Override
+  public boolean hasFlag(int flag) throws RemoteException
+  {
+    return (this.getFlags() & flag) == flag;
+  }
+  
+  /**
+   * @see de.willuhn.jameica.hbci.rmi.Flaggable#getFlags()
+   */
+  @Override
+  public int getFlags() throws RemoteException
+  {
+    Integer i = (Integer) this.getAttribute("flags");
+    return i == null ? Konto.FLAG_NONE : i.intValue();
+  }
+  
+  /**
+   * @see de.willuhn.jameica.hbci.rmi.Flaggable#setFlags(int)
+   */
+  @Override
+  public void setFlags(int flags) throws RemoteException
+  {
+    if (flags < 0)
+      return; // ungueltig
+    
+    this.setAttribute("flags", Integer.valueOf(flags));
+  }
 }
-
-/*******************************************************************************
- * $Log: UmsatzTypImpl.java,v $
- * Revision 1.66  2012/04/05 21:44:18  willuhn
- * @B BUGZILLA 1219
- *
- * Revision 1.65  2012/01/02 22:32:20  willuhn
- * @N BUGZILLA 1170
- *
- * Revision 1.64  2011-07-20 15:41:36  willuhn
- * @N Neue Funktion UmsatzTyp#matches(Umsatz,boolean allowReassign) - normalerweise liefert die Funktion ohne das Boolean false, wenn der Umsatz bereits manuell einer anderen Kategorie zugeordnet ist. Andernfalls kaeme es hier ja - zumindest virtuell - zu einer Doppel-Zuordnung. Da "UmsatzList" jedoch fuer den Suchbegriff (den man oben eingeben kann) intern on-the-fly einen UmsatzTyp erstellt, mit dem die Suche erfolgt, wuerden hier bereits fest zugeordnete Umsaetze nicht mehr gefunden werden. Daher die neue Funktion.
- *
- * Revision 1.63  2011-04-27 11:07:02  willuhn
- * @B Umsaetze, die bereits fest zugeordnet sind, duerfen nicht gematcht werden
- *
- * Revision 1.62  2011-01-20 17:13:21  willuhn
- * @C HBCIProperties#startOfDay und HBCIProperties#endOfDay nach Jameica in DateUtil verschoben
- *
- * Revision 1.61  2011-01-10 22:29:24  willuhn
- * @B BUGZILLA 977
- *
- * Revision 1.60  2010-12-14 12:47:59  willuhn
- * @B Cache wurde nicht immer korrekt aktualisiert, was dazu fuehren konnte, dass sich das Aendern/Loeschen/Anlegen von Kategorien erst nach 10 Sekunden auswirkte und bis dahin Umsaetze der Kategorie "nicht zugeordnet" zugewiesen wurden, obwohl sie in einer Kategorie waren
- *
- * Revision 1.59  2010-12-14 11:54:08  willuhn
- * *** empty log message ***
- *
- * Revision 1.58  2010-12-07 11:10:33  willuhn
- * @C Verwendungszweck beim Matching mergen
- * @N Pattern-Cache
- *
- * Revision 1.57  2010-11-24 16:27:17  willuhn
- * @R Eclipse BIRT komplett rausgeworden. Diese unsaegliche Monster ;)
- * @N Stattdessen verwenden wir jetzt SWTChart (http://www.swtchart.org). Das ist statt den 6MB von BIRT sagenhafte 250k gross
- *
- * Revision 1.56  2010-08-26 12:53:08  willuhn
- * @N Cache nur befuellen, wenn das explizit gefordert wird. Andernfalls wuerde der Cache u.U. unnoetig gefuellt werden, obwohl nur ein Objekt daraus geloescht werden soll
- *
- * Revision 1.55  2010-08-26 11:31:23  willuhn
- * @N Neuer Cache. In dem werden jetzt die zugeordneten Konten von Auftraegen und Umsaetzen zwischengespeichert sowie die Umsatz-Kategorien. Das beschleunigt das Laden der Umsaetze und Auftraege teilweise erheblich
- *
- * Revision 1.54  2010/06/02 15:32:03  willuhn
- * @N Unique-Constraint auf Spalte "name" in Tabelle "umsatztyp" entfernt. Eine Kategorie kann jetzt mit gleichem Namen beliebig oft auftreten
- * @N Auswahlbox der Oberkategorie in Einstellungen->Umsatz-Kategorien zeigt auch die gleiche Baumstruktur wie bei der Zuordnung der Kategorie in der Umsatzliste
- *
- * Revision 1.53  2010/04/11 20:56:53  willuhn
- * @N BUGZILLA #846
- *
- * Revision 1.52  2010/03/05 20:16:44  willuhn
- * @B ClassCastException - siehe BUGZILLA 686
- *
- * Revision 1.51  2010/03/05 17:54:13  willuhn
- * @C Umsatz-Kategorien nach Nummer und anschliessend nach Name sortieren
- ******************************************************************************/
\ No newline at end of file
